@@ -1,21 +1,6 @@
-# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
-
-# Copyright 2014-2021 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# SPDX-FileCopyrightText: Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
-# This file is part of qutebrowser.
-#
-# qutebrowser is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# qutebrowser is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with qutebrowser.  If not, see <https://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-or-later
 
 """Initialization of qutebrowser and application-wide things.
 
@@ -41,16 +26,18 @@ import os
 import sys
 import functools
 import tempfile
+import pathlib
 import datetime
 import argparse
-from typing import Iterable, Optional, cast
+from typing import Optional
+from collections.abc import Iterable
 
-from PyQt5.QtWidgets import QApplication, QWidget
-from PyQt5.QtGui import QDesktopServices, QPixmap, QIcon
-from PyQt5.QtCore import pyqtSlot, QUrl, QObject, QEvent, pyqtSignal, Qt
+from qutebrowser.qt import machinery
+from qutebrowser.qt.widgets import QApplication, QWidget
+from qutebrowser.qt.gui import QDesktopServices, QPixmap, QIcon
+from qutebrowser.qt.core import pyqtSlot, QUrl, QObject, QEvent, pyqtSignal, Qt
 
 import qutebrowser
-import qutebrowser.resources
 from qutebrowser.commands import runners
 from qutebrowser.config import (config, websettings, configfiles, configinit,
                                 qtargs)
@@ -64,19 +51,18 @@ from qutebrowser.keyinput import macros, eventfilter
 from qutebrowser.mainwindow import mainwindow, prompt, windowundo
 from qutebrowser.misc import (ipc, savemanager, sessions, crashsignal,
                               earlyinit, sql, cmdhistory, backendproblem,
-                              objects, quitter)
+                              objects, quitter, nativeeventfilter)
 from qutebrowser.utils import (log, version, message, utils, urlutils, objreg,
-                               usertypes, standarddir, error, qtutils, debug)
+                               resources, usertypes, standarddir,
+                               error, qtutils, debug)
 # pylint: disable=unused-import
 # We import those to run the cmdutils.register decorators.
 from qutebrowser.mainwindow.statusbar import command
 from qutebrowser.misc import utilcmds
+from qutebrowser.browser import commands
 # pylint: enable=unused-import
 
 
-q_app = cast(QApplication, None)
-
-
 def run(args):
     """Initialize everything and run the application."""
     if args.temp_basedir:
@@ -86,7 +72,7 @@ def run(args):
 
     log.init.debug("Initializing directories...")
     standarddir.init(args)
-    utils.preload_resources()
+    resources.preload()
 
     log.init.debug("Initializing config...")
     configinit.early_init(args)
@@ -119,13 +105,14 @@ def run(args):
             log.init.warning(
                 "Backend from the running instance will be used")
         sys.exit(usertypes.Exit.ok)
-    else:
-        quitter.instance.shutting_down.connect(server.shutdown)
-        server.got_args.connect(lambda args, target_arg, cwd:
-                                process_pos_args(args, cwd=cwd, via_ipc=True,
-                                                 target_arg=target_arg))
 
     init(args=args)
+
+    quitter.instance.shutting_down.connect(server.shutdown)
+    server.got_args.connect(
+        lambda args, target_arg, cwd:
+        process_pos_args(args, cwd=cwd, via_ipc=True, target_arg=target_arg))
+
     ret = qt_mainloop()
     return ret
 
@@ -146,10 +133,12 @@ def init(*, args: argparse.Namespace) -> None:
     crashsignal.crash_handler.init_faulthandler()
 
     objects.qapp.setQuitOnLastWindowClosed(False)
+    # WORKAROUND for KDE file dialogs / QEventLoopLocker quitting:
+    # https://bugreports.qt.io/browse/QTBUG-124386
+    objects.qapp.setQuitLockEnabled(False)
     quitter.instance.shutting_down.connect(QApplication.closeAllWindows)
 
     _init_icon()
-    _init_pulseaudio()
 
     loader.init()
     loader.load_components()
@@ -181,8 +170,9 @@ def _init_icon():
     """Initialize the icon of qutebrowser."""
     fallback_icon = QIcon()
     for size in [16, 24, 32, 48, 64, 96, 128, 256, 512]:
-        filename = ':/icons/qutebrowser-{size}x{size}.png'.format(size=size)
-        pixmap = QPixmap(filename)
+        filename = 'icons/qutebrowser-{size}x{size}.png'.format(size=size)
+        pixmap = QPixmap()
+        pixmap.loadFromData(resources.read_file_binary(filename))
         if pixmap.isNull():
             log.init.warning("Failed to load {}".format(filename))
         else:
@@ -194,27 +184,12 @@ def _init_icon():
         objects.qapp.setWindowIcon(icon)
 
 
-def _init_pulseaudio():
-    """Set properties for PulseAudio.
-
-    WORKAROUND for https://bugreports.qt.io/browse/QTBUG-85363
-
-    Affected Qt versions:
-    - Older than 5.11 (which is unsupported)
-    - 5.14.0 to 5.15.0 (inclusive)
-
-    However, we set this on all versions so that qutebrowser's icon gets picked
-    up as well.
-    """
-    for prop in ['application.name', 'application.icon_name']:
-        os.environ['PULSE_PROP_OVERRIDE_' + prop] = 'qutebrowser'
-
-
 def _process_args(args):
     """Open startpage etc. and process commandline args."""
     if not args.override_restore:
         sessions.load_default(args.session)
 
+    new_window = None
     if not sessions.session_manager.did_load:
         log.init.debug("Initializing main window...")
         private = args.target == 'private-window'
@@ -225,15 +200,17 @@ def _process_args(args):
             error.handle_fatal_exc(err, 'Cannot start in private mode',
                                    no_err_windows=args.no_err_windows)
             sys.exit(usertypes.Exit.err_init)
-        window = mainwindow.MainWindow(private=private)
-        if not args.nowindow:
-            window.show()
-        objects.qapp.setActiveWindow(window)
+
+        new_window = mainwindow.MainWindow(private=private)
 
     process_pos_args(args.command)
     _open_startpage()
     _open_special_pages(args)
 
+    if new_window is not None and not args.nowindow:
+        new_window.show()
+        objects.qapp.setActiveWindow(new_window)
+
     delta = datetime.datetime.now() - earlyinit.START_TIME
     log.init.debug("Init finished after {}s".format(delta.total_seconds()))
 
@@ -257,26 +234,30 @@ def process_pos_args(args, via_ipc=False, cwd=None, target_arg=None):
     if command_target in {'window', 'private-window'}:
         command_target = 'tab-silent'
 
-    win_id: Optional[int] = None
+    window: Optional[mainwindow.MainWindow] = None
 
     if via_ipc and (not args or args == ['']):
-        win_id = mainwindow.get_window(via_ipc=via_ipc,
-                                       target=new_window_target)
-        _open_startpage(win_id)
+        window = mainwindow.get_window(via_ipc=via_ipc, target=new_window_target)
+        _open_startpage(window)
+        window.show()
+        window.maybe_raise()
         return
 
     for cmd in args:
         if cmd.startswith(':'):
-            if win_id is None:
-                win_id = mainwindow.get_window(via_ipc=via_ipc,
-                                               target=command_target)
+            if window is None:
+                window = mainwindow.get_window(via_ipc=via_ipc, target=command_target)
+                # FIXME preserving old behavior, but we probably shouldn't be
+                # doing this...
+                # See https://github.com/qutebrowser/qutebrowser/issues/5094
+                window.maybe_raise()
+
             log.init.debug("Startup cmd {!r}".format(cmd))
-            commandrunner = runners.CommandRunner(win_id)
+            commandrunner = runners.CommandRunner(window.win_id)
             commandrunner.run_safely(cmd[1:])
         elif not cmd:
             log.init.debug("Empty argument")
-            win_id = mainwindow.get_window(via_ipc=via_ipc,
-                                           target=new_window_target)
+            window = mainwindow.get_window(via_ipc=via_ipc, target=new_window_target)
         else:
             if via_ipc and target_arg and target_arg != 'auto':
                 open_target = target_arg
@@ -290,7 +271,7 @@ def process_pos_args(args, via_ipc=False, cwd=None, target_arg=None):
                 message.error("Error in startup argument '{}': {}".format(
                     cmd, e))
             else:
-                win_id = open_url(url, target=open_target, via_ipc=via_ipc)
+                window = open_url(url, target=open_target, via_ipc=via_ipc)
 
 
 def open_url(url, target=None, no_raise=False, via_ipc=True):
@@ -303,39 +284,37 @@ def open_url(url, target=None, no_raise=False, via_ipc=True):
         via_ipc: Whether the arguments were transmitted over IPC.
 
     Return:
-        ID of a window that was used to open URL
+        The MainWindow of a window that was used to open the URL.
     """
     target = target or config.val.new_instance_open_target
     background = target in {'tab-bg', 'tab-bg-silent'}
-    win_id = mainwindow.get_window(via_ipc=via_ipc, target=target,
-                                   no_raise=no_raise)
-    tabbed_browser = objreg.get('tabbed-browser', scope='window',
-                                window=win_id)
+    window = mainwindow.get_window(via_ipc=via_ipc, target=target, no_raise=no_raise)
     log.init.debug("About to open URL: {}".format(url.toDisplayString()))
-    tabbed_browser.tabopen(url, background=background, related=False)
-    return win_id
+    window.tabbed_browser.tabopen(url, background=background, related=False)
+    window.show()
+    window.maybe_raise()
+    return window
 
 
-def _open_startpage(win_id=None):
+def _open_startpage(window: Optional[mainwindow.MainWindow] = None) -> None:
     """Open startpage.
 
     The startpage is never opened if the given windows are not empty.
 
     Args:
-        win_id: If None, open startpage in all empty windows.
+        window: If None, open startpage in all empty windows.
                 If set, open the startpage in the given window.
     """
-    if win_id is not None:
-        window_ids: Iterable[int] = [win_id]
+    if window is not None:
+        windows: Iterable[mainwindow.MainWindow] = [window]
     else:
-        window_ids = objreg.window_registry
-    for cur_win_id in list(window_ids):  # Copying as the dict could change
-        tabbed_browser = objreg.get('tabbed-browser', scope='window',
-                                    window=cur_win_id)
-        if tabbed_browser.widget.count() == 0:
+        windows = objreg.window_registry.values()
+
+    for cur_window in list(windows):  # Copying as the dict could change
+        if cur_window.tabbed_browser.widget.count() == 0:
             log.init.debug("Opening start pages")
             for url in config.val.url.start_pages:
-                tabbed_browser.tabopen(url)
+                cur_window.tabbed_browser.tabopen(url)
 
 
 def _open_special_pages(args):
@@ -352,7 +331,7 @@ def _open_special_pages(args):
     tabbed_browser = objreg.get('tabbed-browser', scope='window',
                                 window='last-focused')
 
-    pages = [
+    pages: list[tuple[str, bool, str]] = [
         # state, condition, URL
         ('quickstart-done',
          True,
@@ -368,8 +347,20 @@ def _open_special_pages(args):
          'qute://warning/webkit'),
 
         ('session-warning-shown',
-         qtutils.version_check('5.15', compiled=False),
+         True,
          'qute://warning/sessions'),
+
+        ('qt5-warning-shown',
+         (
+             machinery.IS_QT5 and
+             machinery.INFO.reason == machinery.SelectionReason.auto and
+             objects.backend != usertypes.Backend.QtWebKit
+         ),
+         'qute://warning/qt5'),
+
+        ('birthday-2023-shown',
+         datetime.date.today() == datetime.date(2023, 12, 14),
+         'https://qutebrowser.org/birthday.html'),
     ]
 
     if 'quickstart-done' not in general_sect:
@@ -395,7 +386,7 @@ def _open_special_pages(args):
         return
 
     try:
-        changelog = utils.read_file('html/doc/changelog.html')
+        changelog = resources.read_file('html/doc/changelog.html')
     except OSError as e:
         log.init.warning(f"Not showing changelog due to {e}")
         return
@@ -431,10 +422,10 @@ def on_focus_changed(_old, new):
 def open_desktopservices_url(url):
     """Handler to open a URL via QDesktopServices."""
     target = config.val.new_instance_open_target
-    win_id = mainwindow.get_window(via_ipc=True, target=target)
-    tabbed_browser = objreg.get('tabbed-browser', scope='window',
-                                window=win_id)
-    tabbed_browser.tabopen(url)
+    window = mainwindow.get_window(via_ipc=True, target=target)
+    window.tabbed_browser.tabopen(url)
+    window.show()
+    window.maybe_raise()
 
 
 # This is effectively a @config.change_filter
@@ -479,11 +470,9 @@ def _init_modules(*, args):
 
     with debug.log_time("init", "Initializing SQL/history"):
         try:
-            log.init.debug("Initializing SQL...")
-            sql.init(os.path.join(standarddir.data(), 'history.sqlite'))
-
             log.init.debug("Initializing web history...")
-            history.init(objects.qapp)
+            history.init(db_path=pathlib.Path(standarddir.data()) / 'history.sqlite',
+                         parent=objects.qapp)
         except sql.KnownError as e:
             error.handle_fatal_exc(e, 'Error initializing SQL',
                                    pre_text='Error initializing SQL',
@@ -492,13 +481,14 @@ def _init_modules(*, args):
 
     log.init.debug("Initializing command history...")
     cmdhistory.init()
-    log.init.debug("Initializing sessions...")
-    sessions.init(objects.qapp)
 
     log.init.debug("Initializing websettings...")
     websettings.init(args)
     quitter.instance.shutting_down.connect(websettings.shutdown)
 
+    log.init.debug("Initializing sessions...")
+    sessions.init(objects.qapp)
+
     if not args.no_err_windows:
         crashsignal.crash_handler.display_faulthandler()
 
@@ -525,6 +515,7 @@ def _init_modules(*, args):
     log.init.debug("Misc initialization...")
     macros.init()
     windowundo.init()
+    nativeeventfilter.init()
 
 
 class Application(QApplication):
@@ -562,11 +553,11 @@ class Application(QApplication):
         log.init.debug("Initializing application...")
 
         self.launch_time = datetime.datetime.now()
-        self.focusObjectChanged.connect(  # type: ignore[attr-defined]
-            self.on_focus_object_changed)
+        self.focusObjectChanged.connect(self.on_focus_object_changed)
 
-        self.setAttribute(Qt.AA_UseHighDpiPixmaps, True)
-        self.setAttribute(Qt.AA_MacDontSwapCtrlAndMeta, True)
+        if machinery.IS_QT5:
+            # default and removed in Qt 6
+            self.setAttribute(Qt.ApplicationAttribute.AA_UseHighDpiPixmaps, True)
 
         self.new_window.connect(self._on_new_window)
 
@@ -578,14 +569,14 @@ class Application(QApplication):
     @pyqtSlot(QObject)
     def on_focus_object_changed(self, obj):
         """Log when the focus object changed."""
-        output = repr(obj)
+        output = qtutils.qobj_repr(obj)
         if self._last_focus_object != output:
             log.misc.debug("Focus object changed: {}".format(output))
         self._last_focus_object = output
 
     def event(self, e):
         """Handle macOS FileOpen events."""
-        if e.type() != QEvent.FileOpen:
+        if e.type() != QEvent.Type.FileOpen:
             return super().event(e)
 
         url = e.url()
