@@ -5,7 +5,8 @@
 """QtWebEngine specific part of the web element API."""
 
 from typing import (
-    TYPE_CHECKING, Any, Callable, Dict, Iterator, Optional, Set, Tuple, Union)
+    TYPE_CHECKING, Any, Optional, Union)
+from collections.abc import Iterator, Callable
 
 from qutebrowser.qt.core import QRect, QEventLoop
 from qutebrowser.qt.widgets import QApplication
@@ -24,11 +25,11 @@ class WebEngineElement(webelem.AbstractWebElement):
 
     _tab: "webenginetab.WebEngineTab"
 
-    def __init__(self, js_dict: Dict[str, Any],
+    def __init__(self, js_dict: dict[str, Any],
                  tab: 'webenginetab.WebEngineTab') -> None:
         super().__init__(tab)
         # Do some sanity checks on the data we get from JS
-        js_dict_types: Dict[str, Union[type, Tuple[type, ...]]] = {
+        js_dict_types: dict[str, Union[type, tuple[type, ...]]] = {
             'id': int,
             'text': str,
             'value': (str, int, float),
@@ -105,7 +106,7 @@ class WebEngineElement(webelem.AbstractWebElement):
         log.stub()
         return QRect()
 
-    def classes(self) -> Set[str]:
+    def classes(self) -> set[str]:
         """Get a list of classes assigned to this element."""
         return set(self._js_dict['class_name'].split())
 
@@ -215,14 +216,16 @@ class WebEngineElement(webelem.AbstractWebElement):
             return False
 
         # Qt 6.3+ needs a user interaction to allow navigations from qute:// to
-        # outside qute:// (like e.g. on qute://bookmarks).
+        # outside qute:// (like e.g. on qute://bookmarks), as well as from file:// to
+        # outside of file:// (e.g. users having a local bookmarks.html).
         versions = version.qtwebengine_versions()
-        if (
-            baseurl.scheme() == "qute" and
-            url.scheme() != "qute" and
-            versions.webengine >= utils.VersionNumber(6, 3)
-        ):
-            return True
+        for scheme in ["qute", "file"]:
+            if (
+                baseurl.scheme() == scheme and
+                url.scheme() != scheme and
+                versions.webengine >= utils.VersionNumber(6, 3)
+            ):
+                return True
 
         return url.scheme() not in urlutils.WEBENGINE_SCHEMES
 
