@@ -1,21 +1,6 @@
-# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
-
-# Copyright 2014-2021 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# SPDX-FileCopyrightText: Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
-# This file is part of qutebrowser.
-#
-# qutebrowser is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# qutebrowser is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with qutebrowser.  If not, see <https://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-or-later
 
 """Functions that return config-related completion models."""
 
@@ -44,16 +29,22 @@ def customized_option(*, info):
 
 def list_option(*, info):
     """A CompletionModel filled with settings whose values are lists."""
-    predicate = lambda opt: (isinstance(info.config.get_obj(opt.name),
-                                        list) and not opt.no_autoconfig)
-    return _option(info, "List options", predicate)
+    return _option(
+        info,
+        "List options",
+        lambda opt: (isinstance(info.config.get_obj(opt.name), list) and
+            not opt.no_autoconfig)
+    )
 
 
 def dict_option(*, info):
     """A CompletionModel filled with settings whose values are dicts."""
-    predicate = lambda opt: (isinstance(info.config.get_obj(opt.name),
-                                        dict) and not opt.no_autoconfig)
-    return _option(info, "Dict options", predicate)
+    return _option(
+        info,
+        "Dict options",
+        lambda opt: (isinstance(info.config.get_obj(opt.name), dict) and
+            not opt.no_autoconfig)
+    )
 
 
 def _option(info, title, predicate):
