@@ -1,21 +1,6 @@
-# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
-
-# Copyright 2014-2021 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# SPDX-FileCopyrightText: Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
-# This file is part of qutebrowser.
-#
-# qutebrowser is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# qutebrowser is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with qutebrowser.  If not, see <https://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-or-later
 
 """Get arguments to pass to Qt."""
 
@@ -23,9 +8,14 @@ import os
 import sys
 import argparse
 import pathlib
-from typing import Any, Dict, Iterator, List, Optional, Sequence, Tuple
+# Using deprecated typing.Callable as a WORKAROUND because
+# collections.abc.Callable inside Optional[...]/Union[...]
+# is broken on Python 3.9.0 and 3.9.1
+from typing import Any, Optional, Union, Callable
+from collections.abc import Iterator, Sequence
 
-from PyQt5.QtCore import QLibraryInfo, QLocale
+from qutebrowser.qt import machinery
+from qutebrowser.qt.core import QLocale
 
 from qutebrowser.config import config
 from qutebrowser.misc import objects
@@ -37,7 +27,7 @@ _DISABLE_FEATURES = '--disable-features='
 _BLINK_SETTINGS = '--blink-settings='
 
 
-def qt_args(namespace: argparse.Namespace) -> List[str]:
+def qt_args(namespace: argparse.Namespace) -> list[str]:
     """Get the Qt QApplication arguments based on an argparse namespace.
 
     Args:
@@ -75,40 +65,47 @@ def qt_args(namespace: argparse.Namespace) -> List[str]:
         log.init.debug("QtWebEngine requested, but unavailable...")
         return argv
 
+    versions = version.qtwebengine_versions(avoid_init=True)
+    if versions.webengine >= utils.VersionNumber(6, 4):
+        # https://codereview.qt-project.org/c/qt/qtwebengine/+/376704
+        argv.insert(1, "--webEngineArgs")
+
     special_prefixes = (_ENABLE_FEATURES, _DISABLE_FEATURES, _BLINK_SETTINGS)
     special_flags = [flag for flag in argv if flag.startswith(special_prefixes)]
     argv = [flag for flag in argv if not flag.startswith(special_prefixes)]
-    argv += list(_qtwebengine_args(namespace, special_flags))
+    argv += list(_qtwebengine_args(versions, namespace, special_flags))
 
     return argv
 
 
-def _qtwebengine_features(
+def _qtwebengine_features(  # noqa: C901
         versions: version.WebEngineVersions,
         special_flags: Sequence[str],
-) -> Tuple[Sequence[str], Sequence[str]]:
+) -> tuple[Sequence[str], Sequence[str]]:
     """Get a tuple of --enable-features/--disable-features flags for QtWebEngine.
 
     Args:
         versions: The WebEngineVersions to get flags for.
         special_flags: Existing flags passed via the commandline.
     """
+    assert versions.chromium_major is not None
+
     enabled_features = []
     disabled_features = []
 
     for flag in special_flags:
         if flag.startswith(_ENABLE_FEATURES):
-            flag = flag[len(_ENABLE_FEATURES):]
+            flag = flag.removeprefix(_ENABLE_FEATURES)
             enabled_features += flag.split(',')
         elif flag.startswith(_DISABLE_FEATURES):
-            flag = flag[len(_DISABLE_FEATURES):]
+            flag = flag.removeprefix(_DISABLE_FEATURES)
             disabled_features += flag.split(',')
         elif flag.startswith(_BLINK_SETTINGS):
             pass
         else:
             raise utils.Unreachable(flag)
 
-    if versions.webengine >= utils.VersionNumber(5, 15, 1) and utils.is_linux:
+    if utils.is_linux:
         # Enable WebRTC PipeWire for screen capturing on Wayland.
         #
         # This is disabled in Chromium by default because of the "dialog hell":
@@ -118,7 +115,7 @@ def _qtwebengine_features(
         # However, we don't have Chromium's confirmation dialog in qutebrowser,
         # so we should only get qutebrowser's permission dialog.
         #
-        # In theory this would be supported with Qt 5.13 already, but
+        # In theory this would be supported with Qt 5.15.0 already, but
         # QtWebEngine only started picking up PipeWire correctly with Qt
         # 5.15.1.
         #
@@ -143,8 +140,8 @@ def _qtwebengine_features(
         if config.val.scrolling.bar == 'overlay':
             enabled_features.append('OverlayScrollbar')
 
-    if (versions.webengine >= utils.VersionNumber(5, 14) and
-            config.val.content.headers.referer == 'same-domain'):
+    if (config.val.content.headers.referer == 'same-domain' and
+            versions.chromium_major < 89):
         # Handling of reduced-referrer-granularity in Chromium 76+
         # https://chromium-review.googlesource.com/c/chromium/src/+/1572699
         #
@@ -157,6 +154,17 @@ def _qtwebengine_features(
         # WORKAROUND for https://bugreports.qt.io/browse/QTBUG-89740
         disabled_features.append('InstalledApp')
 
+    if versions.webengine >= utils.VersionNumber(6, 7):
+        # WORKAROUND for https://bugreports.qt.io/browse/QTBUG-132681
+        # TODO adjust if fixed in Qt 6.9.2+
+        disabled_features.append('DocumentPictureInPictureAPI')
+
+    if versions.webengine >= utils.VersionNumber(6, 9):
+        # WORKAROUND for https://bugreports.qt.io/browse/QTBUG-135787
+        # and https://bugreports.qt.io/browse/QTBUG-141096
+        # TODO adjust if fixed in Qt 6.9.2+
+        disabled_features.append('PermissionElement')
+
     if not config.val.input.media_keys:
         disabled_features.append('HardwareMediaKeyHandling')
 
@@ -174,7 +182,7 @@ def _get_pak_name(locale_name: str) -> str:
     Based on Chromium's behavior in l10n_util::CheckAndResolveLocale:
     https://source.chromium.org/chromium/chromium/src/+/master:ui/base/l10n/l10n_util.cc;l=344-428;drc=43d5378f7f363dab9271ca37774c71176c9e7b69
     """
-    if locale_name in {'en', 'en-PH', 'en-LR'}:
+    if locale_name in {'en', 'en-POSIX', 'en-PH', 'en-LR'}:
         return 'en-US'
     elif locale_name.startswith('en-'):
         return 'en-GB'
@@ -199,7 +207,7 @@ def _webengine_locales_path() -> pathlib.Path:
         # not QtWebEngine.
         base = pathlib.Path('/app/translations')
     else:
-        base = pathlib.Path(QLibraryInfo.location(QLibraryInfo.TranslationsPath))
+        base = qtutils.library_path(qtutils.LibraryPath.translations)
     return base / 'qtwebengine_locales'
 
 
@@ -210,7 +218,7 @@ def _get_lang_override(
     """Get a --lang switch to override Qt's locale handling.
 
     This is needed as a WORKAROUND for https://bugreports.qt.io/browse/QTBUG-91715
-    There is no fix yet, but we assume it'll be fixed with QtWebEngine 5.15.4.
+    Fixed with QtWebEngine 5.15.4.
     """
     if not config.val.qt.workarounds.locale:
         return None
@@ -239,30 +247,15 @@ def _get_lang_override(
 
 
 def _qtwebengine_args(
+        versions: version.WebEngineVersions,
         namespace: argparse.Namespace,
         special_flags: Sequence[str],
 ) -> Iterator[str]:
     """Get the QtWebEngine arguments to use based on the config."""
-    versions = version.qtwebengine_versions(avoid_init=True)
-
-    qt_514_ver = utils.VersionNumber(5, 14)
-    qt_515_ver = utils.VersionNumber(5, 15)
-    if qt_514_ver <= versions.webengine < qt_515_ver:
-        # WORKAROUND for https://bugreports.qt.io/browse/QTBUG-82105
-        yield '--disable-shared-workers'
-
-    # WORKAROUND equivalent to
     # https://codereview.qt-project.org/c/qt/qtwebengine/+/256786
-    # also see:
     # https://codereview.qt-project.org/c/qt/qtwebengine-chromium/+/265753
-    if versions.webengine >= utils.VersionNumber(5, 12, 3):
-        if 'stack' in namespace.debug_flags:
-            # Only actually available in Qt 5.12.5, but let's save another
-            # check, as passing the option won't hurt.
-            yield '--enable-in-process-stack-traces'
-    else:
-        if 'stack' not in namespace.debug_flags:
-            yield '--disable-in-process-stack-traces'
+    if 'stack' in namespace.debug_flags:
+        yield '--enable-in-process-stack-traces'
 
     lang_override = _get_lang_override(
         webengine_version=versions.webengine,
@@ -298,73 +291,92 @@ def _qtwebengine_args(
     yield from _qtwebengine_settings_args(versions)
 
 
+_SettingValueType = Union[
+    str,
+    Callable[
+        [
+            version.WebEngineVersions,
+        ],
+        Optional[str],
+    ],
+]
+_WEBENGINE_SETTINGS: dict[str, dict[Any, Optional[_SettingValueType]]] = {
+    'qt.force_software_rendering': {
+        'software-opengl': None,
+        'qt-quick': None,
+        'chromium': '--disable-gpu',
+        'none': None,
+    },
+    'content.canvas_reading': {
+        True: None,
+        # might be overridden in webenginesettings.py
+        False: '--disable-reading-from-canvas',
+    },
+    'content.webrtc_ip_handling_policy': {
+        'all-interfaces': None,
+        'default-public-and-private-interfaces':
+            '--force-webrtc-ip-handling-policy='
+            'default_public_and_private_interfaces',
+        'default-public-interface-only':
+            '--force-webrtc-ip-handling-policy='
+            'default_public_interface_only',
+        'disable-non-proxied-udp':
+            '--force-webrtc-ip-handling-policy='
+            'disable_non_proxied_udp',
+    },
+    'content.javascript.legacy_touch_events': {
+        'always': '--touch-events=enabled',
+        'auto': '--touch-events=auto',
+        'never': '--touch-events=disabled',
+    },
+    'qt.chromium.process_model': {
+        'process-per-site-instance': None,
+        'process-per-site': '--process-per-site',
+        'single-process': '--single-process',
+    },
+    'qt.chromium.low_end_device_mode': {
+        'auto': None,
+        'always': '--enable-low-end-device-mode',
+        'never': '--disable-low-end-device-mode',
+    },
+    'content.prefers_reduced_motion': {
+        True: '--force-prefers-reduced-motion',
+        False: None,
+    },
+    'qt.chromium.sandboxing': {
+        'enable-all': None,
+        'disable-seccomp-bpf': '--disable-seccomp-filter-sandbox',
+        'disable-all': '--no-sandbox',
+    },
+    'qt.chromium.experimental_web_platform_features': {
+        'always': '--enable-experimental-web-platform-features',
+        'never': None,
+        'auto':
+            '--enable-experimental-web-platform-features' if machinery.IS_QT5 else None,
+    },
+    'qt.workarounds.disable_accelerated_2d_canvas': {
+        'always': '--disable-accelerated-2d-canvas',
+        'never': None,
+        'auto': lambda versions: '--disable-accelerated-2d-canvas'
+        if machinery.IS_QT6
+        and versions.webengine
+        and versions.webengine < utils.VersionNumber(6, 8, 2)
+        else None,
+    },
+}
+
+
 def _qtwebengine_settings_args(versions: version.WebEngineVersions) -> Iterator[str]:
-    settings: Dict[str, Dict[Any, Optional[str]]] = {
-        'qt.force_software_rendering': {
-            'software-opengl': None,
-            'qt-quick': None,
-            'chromium': '--disable-gpu',
-            'none': None,
-        },
-        'content.canvas_reading': {
-            True: None,
-            False: '--disable-reading-from-canvas',
-        },
-        'content.webrtc_ip_handling_policy': {
-            'all-interfaces': None,
-            'default-public-and-private-interfaces':
-                '--force-webrtc-ip-handling-policy='
-                'default_public_and_private_interfaces',
-            'default-public-interface-only':
-                '--force-webrtc-ip-handling-policy='
-                'default_public_interface_only',
-            'disable-non-proxied-udp':
-                '--force-webrtc-ip-handling-policy='
-                'disable_non_proxied_udp',
-        },
-        'qt.process_model': {
-            'process-per-site-instance': None,
-            'process-per-site': '--process-per-site',
-            'single-process': '--single-process',
-        },
-        'qt.low_end_device_mode': {
-            'auto': None,
-            'always': '--enable-low-end-device-mode',
-            'never': '--disable-low-end-device-mode',
-        },
-        'content.headers.referer': {
-            'always': None,
-        }
-    }
-    qt_514_ver = utils.VersionNumber(5, 14)
-
-    if qt_514_ver <= versions.webengine < utils.VersionNumber(5, 15, 2):
-        # In Qt 5.14 to 5.15.1, `--force-dark-mode` is used to set the
-        # preferred colorscheme. In Qt 5.15.2, this is handled by a
-        # blink-setting in browser/webengine/darkmode.py instead.
-        settings['colors.webpage.preferred_color_scheme'] = {
-            'dark': '--force-dark-mode',
-            'light': None,
-            'auto': None,
-        }
-
-    referrer_setting = settings['content.headers.referer']
-    if versions.webengine >= qt_514_ver:
-        # Starting with Qt 5.14, this is handled via --enable-features
-        referrer_setting['same-domain'] = None
-    else:
-        referrer_setting['same-domain'] = '--reduced-referrer-granularity'
-
-    # WORKAROUND for https://bugreports.qt.io/browse/QTBUG-60203
-    can_override_referer = (
-        versions.webengine >= utils.VersionNumber(5, 12, 4) and
-        versions.webengine != utils.VersionNumber(5, 13)
-    )
-    referrer_setting['never'] = None if can_override_referer else '--no-referrers'
-
-    for setting, args in sorted(settings.items()):
+    for setting, args in sorted(_WEBENGINE_SETTINGS.items()):
         arg = args[config.instance.get(setting)]
-        if arg is not None:
+        if callable(arg):
+            result = arg(versions)
+            if result is not None:
+                assert isinstance(
+                    result, str
+                ), f"qt.settings feature detection returned an invalid type: {type(result)} for {setting}"
+                yield result
+        elif arg is not None:
             yield arg
 
 
@@ -403,10 +415,7 @@ def init_envvars() -> None:
         os.environ['QT_WAYLAND_DISABLE_WINDOWDECORATION'] = '1'
 
     if config.val.qt.highdpi:
-        env_var = ('QT_ENABLE_HIGHDPI_SCALING'
-                   if qtutils.version_check('5.14', compiled=False)
-                   else 'QT_AUTO_SCREEN_SCALE_FACTOR')
-        os.environ[env_var] = '1'
+        os.environ['QT_ENABLE_HIGHDPI_SCALING'] = '1'
 
     for var, val in config.val.qt.environ.items():
         if val is None and var in os.environ:
