@@ -1,23 +1,9 @@
 #!/usr/bin/env python3
-# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2014-2021 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
-# Copyright 2014-2018 Claude (longneck) <longneck@scratchbook.ch>
-
-# This file is part of qutebrowser.
+# SPDX-FileCopyrightText: Claude (longneck) <longneck@scratchbook.ch>
+# SPDX-FileCopyrightText: Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
-# qutebrowser is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# qutebrowser is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with qutebrowser.  If not, see <https://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-or-later
 
 
 """Tool to import data from other browsers.
@@ -26,7 +12,7 @@ Currently importing bookmarks from Netscape HTML Bookmark files, Chrome
 profiles, and Mozilla profiles is supported.
 """
 
-
+import contextlib
 import argparse
 import textwrap
 import sqlite3
@@ -223,7 +209,7 @@ def import_html_bookmarks(bookmarks_file, bookmark_types, output_format):
     }
     bookmarks = []
     for typ in bookmark_types:
-        tags = soup.findAll(bookmark_query[typ])
+        tags = soup.find_all(bookmark_query[typ])
         for tag in tags:
             if typ == 'search':
                 tag['href'] = search_escape(tag['href']).replace('%s', '{}')
@@ -281,14 +267,15 @@ def import_moz_places(profile, bookmark_types, output_format):
     def search_conv(url):
         return search_escape(url).replace('%s', '{}')
 
-    places = sqlite3.connect(os.path.join(profile, "places.sqlite"))
-    places.create_function('search_conv', 1, search_conv)
-    places.row_factory = sqlite3.Row
-    c = places.cursor()
-    for typ in bookmark_types:
-        c.execute(place_query[typ])
-        for row in c:
-            print(out_template[output_format][typ].format(**row))
+    places_sqlite = os.path.join(profile, "places.sqlite")
+    with contextlib.closing(sqlite3.connect(places_sqlite)) as places:
+        places.create_function('search_conv', 1, search_conv)
+        places.row_factory = sqlite3.Row
+        c = places.cursor()
+        for typ in bookmark_types:
+            c.execute(place_query[typ])
+            for row in c:
+                print(out_template[output_format][typ].format(**row))
 
 
 def import_chrome(profile, bookmark_types, output_format):
@@ -306,17 +293,18 @@ def import_chrome(profile, bookmark_types, output_format):
     }
 
     if 'search' in bookmark_types:
-        webdata = sqlite3.connect(os.path.join(profile, 'Web Data'))
-        c = webdata.cursor()
-        c.execute('SELECT keyword,url FROM keywords;')
-        for keyword, url in c:
-            try:
-                url = opensearch_convert(url)
-                print(out_template[output_format].format(
-                    keyword=keyword, url=url))
-            except KeyError:
-                print('# Unsupported parameter in url for {}; skipping....'.
-                      format(keyword))
+        webdata_db = os.path.join(profile, 'Web Data')
+        with contextlib.closing(sqlite3.connect(webdata_db)) as webdata:
+            c = webdata.cursor()
+            c.execute('SELECT keyword,url FROM keywords;')
+            for keyword, url in c:
+                try:
+                    url = opensearch_convert(url)
+                    print(out_template[output_format].format(
+                        keyword=keyword, url=url))
+                except KeyError:
+                    print('# Unsupported parameter in url for {}; skipping....'.
+                        format(keyword))
 
     else:
         with open(os.path.join(profile, 'Bookmarks'), encoding='utf-8') as f:
