@@ -132,10 +132,10 @@ def _apply_platform_markers(config, item):
             (
                 config.webengine
                 and version.qtwebengine_versions(avoid_init=True).webengine
-                == utils.VersionNumber(6, 9)
+                > utils.VersionNumber(6, 9)
                 and testutils.ON_CI
             ),
-            "Flaky with QtWebEngine 6.9 on CI",
+            "Flaky with QtWebEngine 6.9+ on CI",
         ),
         (
             "qt69_ci_skip",  # WORKAROUND: https://github.com/qutebrowser/qutebrowser/issues/8444#issuecomment-2569610110
@@ -143,10 +143,10 @@ def _apply_platform_markers(config, item):
             (
                 config.webengine
                 and version.qtwebengine_versions(avoid_init=True).webengine
-                == utils.VersionNumber(6, 9)
+                > utils.VersionNumber(6, 9)
                 and testutils.ON_CI
             ),
-            "Skipped with QtWebEngine 6.9 on CI",
+            "Skipped with QtWebEngine 6.9+ on CI",
         ),
     ]
 
