@@ -8,6 +8,7 @@ import sys
 import logging
 import types
 import enum
+import inspect
 import textwrap
 
 import pytest
@@ -440,7 +441,15 @@ class TestArgument:
 
     def test_no_docstring_with_optimize(self, monkeypatch):
         """With -OO we'd get a warning on start, but no warning afterwards."""
-        monkeypatch.setattr(sys, 'flags', types.SimpleNamespace(optimize=2))
+        sys_flags_fake = types.SimpleNamespace(
+            **{
+                k: v
+                for k, v in inspect.getmembers(sys.flags)
+                if not k.startswith("_") and k not in {"count", "index"}
+            }
+        )
+        sys_flags_fake.optimize = 2
+        monkeypatch.setattr(sys, 'flags', sys_flags_fake)
 
         @cmdutils.register()
         def fun():
