@@ -1,22 +1,8 @@
-# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
+# SPDX-FileCopyrightText: Michal Siedlaczek <michal.siedlaczek@gmail.com>
+# SPDX-FileCopyrightText: Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+#
+# SPDX-License-Identifier: GPL-3.0-or-later
 
-# Copyright 2017-2021 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
-# Copyright 2017-2018 Michal Siedlaczek <michal.siedlaczek@gmail.com>
-
-# This file is part of qutebrowser.
-#
-# qutebrowser is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# qutebrowser is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with qutebrowser.  If not, see <https://www.gnu.org/licenses/>.
 
 import logging
 import os
@@ -34,7 +20,7 @@ def test_version(message_mock, caplog):
     with caplog.at_level(logging.WARNING):
         assert spell.version('malformed_filename') is None
     msg = message_mock.getmsg(usertypes.MessageLevel.warning)
-    expected = ("Found a dictionary with a malformed name: malformed_filename")
+    expected = "Found a dictionary with a malformed name: malformed_filename"
     assert msg.text == expected
 
 
