@@ -5,7 +5,6 @@
 
 import os
 import dataclasses
-from typing import List
 
 import pytest
 import bs4
@@ -101,8 +100,8 @@ class TestDirbrowserHtml:
     class Parsed:
 
         parent: str
-        folders: List[str]
-        files: List[str]
+        folders: list[str]
+        files: list[str]
 
     @dataclasses.dataclass
     class Item:
