@@ -1,28 +1,13 @@
-# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
-
-# Copyright 2015-2021 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# SPDX-FileCopyrightText: Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
-# This file is part of qutebrowser.
-#
-# qutebrowser is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# qutebrowser is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with qutebrowser.  If not, see <https://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-or-later
 
 """Tests for browser.network.networkreply."""
 
 import pytest
 
-from PyQt5.QtCore import QUrl, QIODevice
-from PyQt5.QtNetwork import QNetworkRequest, QNetworkReply
+from qutebrowser.qt.core import QUrl, QIODevice
+from qutebrowser.qt.network import QNetworkRequest, QNetworkReply
 
 from qutebrowser.browser.webkit.network import networkreply
 
@@ -38,11 +23,11 @@ class TestFixedDataNetworkReply:
         reply = networkreply.FixedDataNetworkReply(req, b'', 'test/foo')
         assert reply.request() == req
         assert reply.url() == req.url()
-        assert reply.openMode() == QIODevice.ReadOnly
-        assert reply.header(QNetworkRequest.ContentTypeHeader) == 'test/foo'
-        http_code = reply.attribute(QNetworkRequest.HttpStatusCodeAttribute)
+        assert reply.openMode() == QIODevice.OpenModeFlag.ReadOnly
+        assert reply.header(QNetworkRequest.KnownHeaders.ContentTypeHeader) == 'test/foo'
+        http_code = reply.attribute(QNetworkRequest.Attribute.HttpStatusCodeAttribute)
         http_reason = reply.attribute(
-            QNetworkRequest.HttpReasonPhraseAttribute)
+            QNetworkRequest.Attribute.HttpReasonPhraseAttribute)
         assert http_code == 200
         assert http_reason == 'OK'
         assert reply.isFinished()
@@ -76,20 +61,20 @@ class TestFixedDataNetworkReply:
 
 def test_error_network_reply(qtbot, req):
     reply = networkreply.ErrorNetworkReply(
-        req, "This is an error", QNetworkReply.UnknownNetworkError)
+        req, "This is an error", QNetworkReply.NetworkError.UnknownNetworkError)
 
-    with qtbot.wait_signals([reply.error, reply.finished], order='strict'):
+    with qtbot.wait_signals([reply.errorOccurred, reply.finished], order='strict'):
         pass
 
     reply.abort()  # shouldn't do anything
     assert reply.request() == req
     assert reply.url() == req.url()
-    assert reply.openMode() == QIODevice.ReadOnly
+    assert reply.openMode() == QIODevice.OpenModeFlag.ReadOnly
     assert reply.isFinished()
     assert not reply.isRunning()
     assert reply.bytesAvailable() == 0
     assert reply.readData(1) == b''
-    assert reply.error() == QNetworkReply.UnknownNetworkError
+    assert reply.error() == QNetworkReply.NetworkError.UnknownNetworkError
     assert reply.errorString() == "This is an error"
 
 
@@ -97,5 +82,5 @@ def test_redirect_network_reply():
     url = QUrl('https://www.example.com/')
     reply = networkreply.RedirectNetworkReply(url)
     assert reply.readData(1) == b''
-    assert reply.attribute(QNetworkRequest.RedirectionTargetAttribute) == url
+    assert reply.attribute(QNetworkRequest.Attribute.RedirectionTargetAttribute) == url
     reply.abort()  # shouldn't do anything
