@@ -1,26 +1,11 @@
-# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
-
-# Copyright 2015-2021 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
-# Copyright 2015-2018 lamarpavel
+# SPDX-FileCopyrightText: lamarpavel
+# SPDX-FileCopyrightText: Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
-# This file is part of qutebrowser.
-#
-# qutebrowser is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# qutebrowser is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with qutebrowser.  If not, see <https://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-or-later
 
 import pytest
-from PyQt5.QtCore import QUrl, QDateTime
-from PyQt5.QtNetwork import QNetworkDiskCache, QNetworkCacheMetaData
+from qutebrowser.qt.core import QUrl, QDateTime
+from qutebrowser.qt.network import QNetworkDiskCache, QNetworkCacheMetaData
 
 from qutebrowser.browser.webkit import cache
 
@@ -30,10 +15,19 @@ def disk_cache(tmpdir, config_stub):
     return cache.DiskCache(str(tmpdir))
 
 
-def preload_cache(cache, url='http://www.example.com/', content=b'foobar'):
+def build_metadata(url='http://qutebrowser.org/'):
     metadata = QNetworkCacheMetaData()
     metadata.setUrl(QUrl(url))
+    # https://codereview.qt-project.org/c/qt/qtbase/+/465547
+    metadata.setRawHeaders([(b"X-Hello", b"World")])
+
     assert metadata.isValid()
+    assert metadata.rawHeaders()
+    return metadata
+
+
+def preload_cache(cache, url='http://www.example.com/', content=b'foobar'):
+    metadata = build_metadata(url)
     device = cache.prepare(metadata)
     assert device is not None
     device.write(content)
@@ -71,12 +65,8 @@ def test_cache_size_leq_max_cache_size(config_stub, tmpdir):
 
 def test_cache_existing_metadata_file(tmpdir, disk_cache):
     """Test querying existing meta data file from activated cache."""
-    url = 'http://qutebrowser.org'
     content = b'foobar'
-
-    metadata = QNetworkCacheMetaData()
-    metadata.setUrl(QUrl(url))
-    assert metadata.isValid()
+    metadata = build_metadata()
 
     device = disk_cache.prepare(metadata)
     assert device is not None
@@ -173,8 +163,7 @@ def test_cache_full(tmpdir):
     content2 = b'ohmycert'
     preload_cache(disk_cache, url2, content2)
 
-    metadata = QNetworkCacheMetaData()
-    metadata.setUrl(QUrl(url))
+    metadata = build_metadata(url)
     soon = QDateTime.currentDateTime().addMonths(4)
     assert soon.isValid()
     metadata.setLastModified(soon)
