@@ -1,21 +1,6 @@
-# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
-
-# Copyright 2019-2020 Jay Kamat <jaygkamat@gmail.com>:
+# SPDX-FileCopyrightText: Jay Kamat <jaygkamat@gmail.com>:
 #
-# This file is part of qutebrowser.
-#
-# qutebrowser is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# qutebrowser is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with qutebrowser.  If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-or-later
 
 """Tests for the BindingTrie."""
 
@@ -25,7 +10,7 @@ import textwrap
 
 import pytest
 
-from PyQt5.QtGui import QKeySequence
+from qutebrowser.qt.gui import QKeySequence
 
 from qutebrowser.keyinput import basekeyparser
 from qutebrowser.keyinput import keyutils
@@ -39,7 +24,7 @@ def test_matches_single(entered, configured, match_type):
     configured = keyutils.KeySequence.parse(configured)
     trie = basekeyparser.BindingTrie()
     trie[configured] = "eeloo"
-    command = "eeloo" if match_type == QKeySequence.ExactMatch else None
+    command = "eeloo" if match_type == QKeySequence.SequenceMatch.ExactMatch else None
     result = basekeyparser.MatchResult(match_type=match_type,
                                        command=command,
                                        sequence=entered)
@@ -82,22 +67,22 @@ def test_str():
 @pytest.mark.parametrize('configured, expected', [
     ([],
      # null match
-     [('a', QKeySequence.NoMatch),
-      ('', QKeySequence.NoMatch)]),
+     [('a', QKeySequence.SequenceMatch.NoMatch),
+      ('', QKeySequence.SequenceMatch.NoMatch)]),
     (['abcd'],
-     [('abcd', QKeySequence.ExactMatch),
-      ('abc', QKeySequence.PartialMatch)]),
+     [('abcd', QKeySequence.SequenceMatch.ExactMatch),
+      ('abc', QKeySequence.SequenceMatch.PartialMatch)]),
     (['aa', 'ab', 'ac', 'ad'],
-     [('ac', QKeySequence.ExactMatch),
-      ('a', QKeySequence.PartialMatch),
-      ('f', QKeySequence.NoMatch),
-      ('acd', QKeySequence.NoMatch)]),
+     [('ac', QKeySequence.SequenceMatch.ExactMatch),
+      ('a', QKeySequence.SequenceMatch.PartialMatch),
+      ('f', QKeySequence.SequenceMatch.NoMatch),
+      ('acd', QKeySequence.SequenceMatch.NoMatch)]),
     (['aaaaaaab', 'aaaaaaac', 'aaaaaaad'],
-     [('aaaaaaab', QKeySequence.ExactMatch),
-      ('z', QKeySequence.NoMatch)]),
+     [('aaaaaaab', QKeySequence.SequenceMatch.ExactMatch),
+      ('z', QKeySequence.SequenceMatch.NoMatch)]),
     (string.ascii_letters,
-     [('a', QKeySequence.ExactMatch),
-      ('!', QKeySequence.NoMatch)]),
+     [('a', QKeySequence.SequenceMatch.ExactMatch),
+      ('!', QKeySequence.SequenceMatch.NoMatch)]),
 ])
 def test_matches_tree(configured, expected, benchmark):
     trie = basekeyparser.BindingTrie()
@@ -107,7 +92,7 @@ def test_matches_tree(configured, expected, benchmark):
     def run():
         for entered, match_type in expected:
             sequence = keyutils.KeySequence.parse(entered)
-            command = ("eeloo" if match_type == QKeySequence.ExactMatch
+            command = ("eeloo" if match_type == QKeySequence.SequenceMatch.ExactMatch
                        else None)
             result = basekeyparser.MatchResult(match_type=match_type,
                                                command=command,
